﻿
(function () {
    "use strict";
    angular.module("enrDetailsPaneModule")
           .directive("enrOffices", [function () {
               return {
                   templateUrl: "Scripts/EnrAngular/EnrDetailsPane/enrOffices.html",
                   controller: "enrOfficesController",
                   scope: { officeName: "=", countyName: "=" },
                   restrict: "E"
               };
           }])

           .controller("enrOfficesController", ["$scope", "$filter", "detailsData", "enrModeManager", "officeJsonRegionFilter", "regionsJsonParser", "participationCalculator", "primaryService", "gradientService","sortDataService",
             function ($scope, $filter, detailsData, enrModeManager, officeJsonRegionFilter, regionsJsonParser, participationCalculator, primaryService, gradientService, sortDataService) {

                 var selectedCountyName = $scope.countyName;
                 var currentOfficeName =  $scope.officeName;
                 $scope.raceList = [];        
                 
                 var refreshDataBindings = function () {
                     var currentOffices = primaryService.getOfficesList().slice();
                     var officeCatJson = {};
                     for (var i = 0; i < currentOffices.length; i++) {
                         if (currentOffices[i].OfficeCategory.OFFICE_CATEGORY_NAME == currentOfficeName) {
                             officeCatJson = currentOffices[i];
                             break;
                         }
                     }
                     
                     //Binding races
                     $scope.raceList = getRegionalRaceListFromOffCatJson(officeCatJson);                     
                 }

                 // Data Population     
                 var getBindableRaceListFromRaceJson = function (raceListJson) {
                     var raceList = [];
                     var isPrimary = primaryService.getIsPrimary();
                     var selectedPartyCode = primaryService.getSelectedParty();

                     for (var i = 0; i < raceListJson.length; i++) {
                         var curRaceJson = raceListJson[i];
                         var newRaceListItem = { raceName: curRaceJson.OFFICE_TITLE, candidateList: [] };

                         var candidateListJson = curRaceJson.Candidates.Candidate;
                         if (candidateListJson.length === undefined)
                             candidateListJson = [candidateListJson];

                         if (isPrimary && !gradientService.isOfficeExcludedFromGradients(currentOfficeName)) {
                             gradientService.generateOfficeCandidateGradients(candidateListJson);
                         }

                         for (var j = 0; j < candidateListJson.length; j++) {
                             var curCandidateJson = candidateListJson[j];
                             var partyCode = curCandidateJson.PARTY_ABBREV;                             

                             // In primary mode, we only show members of the currently selected party.
                             if (isPrimary && partyCode !== selectedPartyCode)
                                 continue;

                             var newCandidateListItem = {
                                 name: curCandidateJson.NAME_ON_BALLOT,
                                 nameLast: curCandidateJson.CandidateName,
                                 partyLetter: partyCode,
                                 partyColor: curCandidateJson.DISPLAY_COLOR,
                                 votes: curCandidateJson.TOTAL_VOTES,
                                 order: sortDataService.getOrdrerNumber(partyCode)
                             }

                             if (isPrimary && !gradientService.isOfficeExcludedFromGradients(currentOfficeName)) {                                

                                 // Depending on the source of the JSON, we may have to manually grab gradient color
                                 if (!curCandidateJson.GRADIENT_COLOR) {
                                     var candidateInfoWithGradients = gradientService.getGradientsByParty(curCandidateJson.DISPLAY_COLOR);
                                     var gradient = $filter("filter")(candidateInfoWithGradients, { NAME_ON_BALLOT: curCandidateJson.NAME_ON_BALLOT })[0];
                                     if (gradient && gradient.GRADIENT_COLOR)
                                         curCandidateJson.GRADIENT_COLOR = gradient.GRADIENT_COLOR;
                                 }

                                 newCandidateListItem.gradientColor = curCandidateJson.GRADIENT_COLOR;
                             }

                             newRaceListItem.candidateList.push(newCandidateListItem);
                         }

                         if (newRaceListItem.candidateList.length > 0) {
                             //newRaceListItem.candidateList = sortCandidatesByVotes(newRaceListItem.candidateList,'votes');
                             //newRaceListItem.candidateList = sortDataService.sortCandiatesByParty(newRaceListItem.candidateList, 'order', 'votes', 'nameLast');
                             raceList.push(newRaceListItem);
                         }
                     }
                   

                     // Percentages
                     for (var i = 0; i < raceList.length; i++) {
                         var curRaceItem = raceList[i];

                         var totalVotes = 0;
                         for (var k = 0; k < curRaceItem.candidateList.length; k++)
                             totalVotes += Number(curRaceItem.candidateList[k].votes);
                         for (var k = 0; k < curRaceItem.candidateList.length; k++)
                             curRaceItem.candidateList[k].percentage = totalVotes !== 0 ? curRaceItem.candidateList[k].votes / totalVotes : 0;
                     }

                     return raceList;
                 }
                 var sortCandidatesByVotes = function (candidateList, field) {                     
                     candidateList.sort(function (a, b) {
                         return (parseInt(a[field]) > parseInt(b[field])) ? 1 : ((parseInt(a[field]) < parseInt(b[field])) ? -1 : 0);
                     });
                     return candidateList.reverse();
                 }
                 var getRegionalRaceListFromOffCatJson = function (offCatJson) {
                     var raceList = [];

                     var isValidRegionsData = regionsJsonParser.validOfficesRegionsJsonExist(offCatJson);
                     if (isValidRegionsData) {
                         var regionsJson = regionsJsonParser.getOfficesRegionsJsonAsArray(offCatJson);
                         for (var i = 0; i < regionsJson.length; i++) {
                             if (regionsJson[i].MAP_JURISDICTION_NAME == selectedCountyName) {                                 
                                 var raceListJson = regionsJson[i].Races.Race;
                                 if (raceListJson.length === undefined)
                                     raceListJson = [raceListJson];

                                 raceList = getBindableRaceListFromRaceJson(raceListJson);
                                 break;
                             }
                         }                        
                     }
                     console.log("test", raceList);
                     return raceList;
                 }


                 refreshDataBindings();
             }]);

}());
